<?php

declare(strict_types=1);

namespace Drupal\webhook\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\webhook\WebhookInterface;

/**
 * Defines the webhook entity class.
 *
 * @ContentEntityType(
 *   id = "webhook",
 *   label = @Translation("Webhook"),
 *   label_collection = @Translation("Webhooks"),
 *   label_singular = @Translation("webhook"),
 *   label_plural = @Translation("webhooks"),
 *   label_count = @PluralTranslation(
 *     singular = "@count webhooks",
 *     plural = "@count webhooks",
 *   ),
 *   bundle_label = @Translation("Webhook type"),
 *   handlers = {
 *     "list_builder" = "Drupal\webhook\WebhookListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "add" = "Drupal\webhook\Form\WebhookForm",
 *       "edit" = "Drupal\webhook\Form\WebhookForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "delete-multiple-confirm" = "Drupal\Core\Entity\Form\DeleteMultipleForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "webhook",
 *   admin_permission = "administer webhook types",
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "bundle",
 *     "label" = "label",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "collection" = "/admin/content/webhook",
 *     "add-form" = "/webhook/add/{webhook_type}",
 *     "add-page" = "/webhook/add",
 *     "canonical" = "/webhook/{webhook}",
 *     "edit-form" = "/webhook/{webhook}/edit",
 *     "delete-form" = "/webhook/{webhook}/delete",
 *     "delete-multiple-form" = "/admin/content/webhook/delete-multiple",
 *   },
 *   bundle_entity_type = "webhook_type",
 *   field_ui_base_route = "entity.webhook_type.edit_form",
 * )
 */
final class Webhook extends ContentEntityBase implements WebhookInterface {

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->get('label')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setLabel($label): Webhook {
    $this->set('label', $label);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp): Webhook {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * Gets payload.
   */
  public function getPayload(): array {
    return (array) json_decode($this->get('payload')->value, TRUE);
  }

  /**
   * Sets payload.
   */
  public function setPayload($payload): Webhook {
    $this->set('payload', $payload);
    return $this;
  }

  /**
   * Gets payload.
   */
  public function getHeaders(): array {
    return (array) json_decode($this->get('headers')->value, TRUE);
  }

  /**
   * Sets payload.
   */
  public function setHeaders($headers): Webhook {
    $this->set('headers', $headers);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Label'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['headers'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Headers'))
      ->setDescription(t('The headers of the webhook entity.'))
      ->setRequired(FALSE)
      ->setDisplayOptions('form', [
        'type' => 'string_textarea',
        'weight' => -5,
        'settings' => [
          'rows' => 12,
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['payload'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Payload'))
      ->setDescription(t('The payload of the webhook entity.'))
      ->setRequired(FALSE)
      ->setDisplayOptions('form', [
        'type' => 'string_textarea',
        'weight' => -5,
        'settings' => [
          'rows' => 12,
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time that the webhook was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 20,
      ])
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
