<?php

declare(strict_types=1);

namespace Drupal\webhook\Form;

use Adbar\Dot;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Webhook form.
 */
final class MappingForm extends FormBase {

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected EntityFieldManager $entityFieldManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected EntityTypeManager $entityTypeManager;

  /**
   * The class constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManager $entity_field_manager
   *   The entity field manager service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(
    EntityFieldManager $entity_field_manager,
    EntityTypeManager $entity_type_manager,
  ) {
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'webhook_mapping';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $webhook_type = $this->getRouteMatch()->getParameter('webhook_type');
    $config = $this->configFactory()->getEditable('webhook.mappings.' . $webhook_type);
    $config->set('mapping', (array) $config->get('mapping'));
    $config->save();

    $mappings = $this->config('webhook.mappings.' . $webhook_type)->get('mapping');
    $fields = $this->entityFieldManager->getFieldDefinitions('webhook', $webhook_type);

    $options = [];
    foreach ($fields as $field) {
      if (empty($mappings[$field->getName()])) {
        $options[$field->getName()] = $field->getLabel() . ' (' . $field->getName() . ')';
      }
    }
    unset($options['id'], $options['uuid'], $options['bundle'], $options['headers'], $options['payload']);

    $form['table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Field'),
        $this->t('Machine name'),
        $this->t('Payload'),
        $this->t('Operations'),
      ],
      '#empty' => $this->t('There are no mapping items yet.'),
    ];
    foreach ((array) $mappings as $field_name => $value) {
      // Some table columns containing raw markup.
      $form['table'][$field_name]['field_label'] = [
        '#plain_text' => isset($fields[$field_name]) ? $fields[$field_name]->getLabel() : $this->t('- Not found -'),
      ];
      $form['table'][$field_name]['field_name'] = [
        '#plain_text' => isset($fields[$field_name]) ? $fields[$field_name]->getName() : $this->t('- Not found -'),
      ];
      $form['table'][$field_name]['value'] = [
        '#plain_text' => $value,
      ];

      // Operations (dropbutton) column.
      $form['table'][$field_name]['operations'] = [
        '#type' => 'operations',
        '#links' => [],
      ];
      $form['table'][$field_name]['operations']['#links']['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('webhook.mapping_delete_confirm', [
          'webhook_type' => $webhook_type,
          'field_name' => $field_name,
        ]),
        'attributes' => [
          'id' => 'my-dialog',
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => Json::encode([
            'width' => 880,
            'title' => $this->t('Delete'),
            'webhook_type' => $webhook_type,
            'field_name' => $field_name,
          ]),
        ],
      ];
    }

    $form['mapping'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Mapping'),
    ];
    $form['mapping']['field_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Field'),
      '#options' => $options,
      '#required' => TRUE,
    ];
    $form['mapping']['mapping_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Payload'),
      '#description' => $this->t('Start getting mapping suggestions from latest webhook <em>header</em> or <em>payload</em> by typing the words. If no suggestions are available, you may need to trigger the webhook first to save the data.'),
      '#autocomplete_route_name' => 'webhook.webhook_autocomplete',
      '#autocomplete_route_parameters' => ['webhook_type' => $webhook_type],
      '#required' => TRUE,
    ];
    $form['mapping']['actions'] = ['#type' => 'actions'];
    $form['mapping']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save mapping'),
    ];

    $webhook = $this->getCurrentWebhook();
    $data = [
      'headers' => $webhook ? $this->getCurrentWebhook()->getHeaders() : '',
      'payload' => $webhook ? $this->getCurrentWebhook()->getPayload() : '',
    ];
    $form['mapping']['tabs'] = [
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-tokens',
      '#title' => $this->t('Payload examples'),
    ];
    $form['mapping']['tokens'] = [
      '#type' => 'details',
      '#title' => $this->t('Tokens'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['webhook-backend-source']],
    ];
    $dot = new Dot($data);
    $form['mapping']['tokens']['payload'] = [
      '#theme' => 'item_list',
      '#list_type' => 'ul',
      '#items' => array_keys($dot->flatten()),
    ];
    $form['mapping']['json'] = [
      '#type' => 'details',
      '#title' => $this->t('Headers & Payload'),
      '#group' => 'tabs',
      '#attributes' => ['class' => ['webhook-backend-source']],
    ];
    $form['mapping']['json']['payload'] = [
      '#markup' => '<pre>' . htmlspecialchars(json_encode($data, JSON_PRETTY_PRINT)) . '</pre>',
    ];
    $form['#attached']['library'][] = 'core/drupal.dialog.ajax';
    $form['#attached']['library'][] = 'webhook/backend';
    $form['mapping']['json']['modal'] = [
      '#markup' => Link::createFromRoute($this->t('Open in modal'), 'webhook.webhook_json', ['webhook_type' => $webhook_type], [
        'attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => json_encode(['height' => 768, 'width' => 1024]),
        ],
      ])->toString(),
    ];
    $form['mapping']['tokens']['modal'] = [
      '#markup' => Link::createFromRoute($this->t('Open in modal'), 'webhook.webhook_token', ['webhook_type' => $webhook_type], [
        'attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => json_encode(['height' => 768, 'width' => 1024]),
        ],
      ])->toString(),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $webhook_type = $this->getRouteMatch()->getParameter('webhook_type');
    $config = $this->configFactory->getEditable('webhook.mappings.' . $webhook_type);

    $mapping = (array) $config->get('mapping');
    $mapping[$form_state->getValue('field_name')] = $form_state->getValue('mapping_value');
    $config->set('mapping', $mapping)->save();

    $this->messenger()->addStatus($this->t('The mapping has been saved.'));
  }

  /**
   * Gets current webhook entity.
   */
  public function getCurrentWebhook() {
    /** @var \Drupal\webhook\Entity\WebhookType $webhook_type */
    $webhook_type = $this->getRouteMatch()->getParameter('webhook_type');
    $entity_query = $this->entityTypeManager->getStorage('webhook')
      ->getQuery();
    $entity_query
      ->condition('bundle', $webhook_type)
      ->accessCheck(FALSE)
      ->sort('created', 'DESC')
      ->range(0, 1);
    if ($ids = $entity_query->execute()) {
      /** @var \Drupal\webhook\Entity\Webhook */
      return $this->entityTypeManager
        ->getStorage('webhook')
        ->load(reset($ids));
    }
    return NULL;
  }

}
