<?php

declare(strict_types=1);

namespace Drupal\webhook\Hook;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Url;
use Drupal\webhook\Entity\WebhookType;

/**
 * Hook implementations for webhooks module.
 */
class WebhookHooks {

  /**
   * Implements hook_entity_operation().
   */
  #[Hook('entity_operation')]
  public function entityOperation(EntityInterface $entity): array {
    $operations = [];
    if ($entity instanceof WebhookType) {
      $operations['mapping'] = [
        'title' => t('Mapping'),
        'url' => Url::fromRoute('webhook.webhook_type.field_mapping', ['webhook_type' => $entity->id()]),
        'weight' => 50,
      ];
    }
    return $operations;
  }

  /**
   * Implements hook_ENTITY_TYPE_insert().
   */
  #[Hook('webhook_config_insert')]
  public function webhookConfigInsert(EntityInterface $entity): void {
    $create_webhook_types = \Drupal::service('config.factory')->get('webhooks.settings')
      ->get('create_webhook_types');
    if ($create_webhook_types && $entity->get('type') === 'incoming') {
      WebhookType::create([
        'id' => $entity->id(),
        'label' => $entity->label(),
      ])->save();
      \Drupal::messenger()->addMessage(t('A Webhook Type has been created for storage. You may now <a href=":fields">add fields</a> and then <a href=":mapping">map the Webhook header and payload values</a> to the fields.', [
        ':fields' => Url::fromRoute('entity.webhook.field_ui_fields', ['webhook_type' => $entity->id()])->toString(),
        ':mapping' => Url::fromRoute('webhook.webhook_type.field_mapping', ['webhook_type' => $entity->id()])->toString(),
      ]));
    }
  }

  /**
   * Implements hook_form_FORM_ID_alter().
   */
  #[Hook('form_webhooks_settings_alter')]
  public function formWebhooksSettingsAlter(&$form, FormStateInterface $form_state, $form_id) : void {
    $form['create_webhook_types'] = [
      '#type' => 'checkbox',
      '#title' => t('Create Webhook types'),
      '#description' => t('Create Webhook types after Webhook config creation for dedicated incoming webhook storage.'),
      '#default_value' => \Drupal::service('config.factory')->get('webhooks.settings')->get('create_webhook_types'),
      '#disabled' => !\Drupal::service('module_handler')->moduleExists('webhook'),
    ];
    $form['actions']['submit']['#submit'][] = '\Drupal\webhook\Hook\WebhookHooks::formWebhooksSettingsSubmit';
  }

  /**
   * Implements form_webhooks_settings_alter submit handler.
   */
  public static function formWebhooksSettingsSubmit(array $form, FormStateInterface $form_state) {
    \Drupal::service('config.factory')->getEditable('webhooks.settings')
      ->set('create_webhook_types', $form_state->getValue('create_webhook_types'))
      ->save();
  }

}
